@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright 2004-2015 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2016 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -82,6 +82,7 @@ library("avformat")
 library("avcodec")
 library("avutil")
 has_avresample = library("avresample", optional=True)
+has_swresample = library("swresample", optional=True)
 has_swscale = library("swscale", optional=True)
 library("freetype")
 has_fribidi = library("fribidi", optional=True)
@@ -130,11 +131,13 @@ if has_fribidi:
 cython("_renpysteam", language="c++", compile_if=steam_sdk, libs=["steam_api"])
 
 # Sound.
-pymodule("pysdlsound.__init__")
 
 sound = [ "avformat", "avcodec", "avutil", "z" ]
 macros = [ ]
 
+if has_swresample:
+    sound.insert(3, "swresample")
+
 if has_avresample:
     sound.insert(0, "avresample")
     macros.append(("HAS_RESAMPLE", 1))
@@ -143,8 +146,8 @@ if has_swscale:
     sound.insert(0, "swscale")
 
 cython(
-    "pysdlsound.sound",
-    [ "pss.c", "ffdecode.c" ],
+    "renpy.audio.renpysound",
+    [ "renpysound_core.c", "ffmedia.c" ],
     libs = sdl + sound,
     define_macros=macros)
 
@@ -157,7 +160,7 @@ cython("renpy.styledata.styleclass")
 cython("renpy.styledata.stylesets")
 
 for p in generate_styles.prefixes:
-    cython("renpy.styledata.style_{}functions".format(p), pyx="gen/style_{}functions.pyx".format(p))
+    cython("renpy.styledata.style_{}functions".format(p), pyx=setuplib.gen + "/style_{}functions.pyx".format(p))
 
 # renpy.display
 cython("renpy.display.render", libs=[ 'z', 'm' ])
