@@ -1,4 +1,4 @@
-# Copyright 2004-2015 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2016 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -26,6 +26,7 @@ import sys
 import pygame_sdl2 as pygame
 import threading
 import renpy.display
+import renpy.audio
 
 
 # Sample surfaces, with and without alpha.
@@ -62,6 +63,7 @@ def set_rgba_masks():
     sample_alpha = pygame.Surface((10, 10), 0, 32, masks)
     sample_noalpha = pygame.Surface((10, 10), 0, 32, masks[:3] + (0,))
 
+    renpy.audio.audio.sample_surfaces(sample_noalpha, sample_alpha)
 
 class Surface(pygame.Surface):
     """
